'''
MIT License

Copyright (c) 2012-2017 Ben Croston <ben@croston.org>
Copyright (c) 2019 NVIDIA CORPORATION
Copyright (c) 2019 Hyeonki Hong <hhk7734@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''

from Odroid._GPIO import *
from Odroid.GPIO import gpio_event as event
import sys

BOARD = 10
BCM = 11
SOC = 100
WIRINGPI = 101

PUD_OFF = 0
PUD_DOWN = 1
PUD_UP = 2

LOW = 0
HIGH = 1

RISING = 1
FALLING = 2
BOTH = 3

UNKNOWN = -1
OUT = 1
IN = 0

_IN_PULL_OFF = IN
_IN_PULL_UP = 2
_IN_PULL_DOWN = 3

_gpio_mode = None

_gpio_warnings = True


def _warning_unsupported_func():
    if _gpio_warnings:
        func_name = str(sys._getframe(1).f_code.co_name)
        print("'{}' function is not yet supported.".format(func_name))


def setmode(mode):
    '''
    Set up numbering mode to use for channels.
    BOARD    - Use Raspberry Pi board numbers.
    BCM      - Use SOC GPIO 00..nn numbers.
    SOC      - Use SOC GPIO 00..nn numbers.
    WIRINGPI - Use wiringPi pin numbers.
    '''
    global _gpio_mode

    # if _gpio_mode and mode != _gpio_mode:
    #     raise ValueError("A different mode has already been set!")

    _gpio_mode = mode

    if mode == BOARD:
        wiringPiSetupPhys()
    elif (mode == BCM) or (mode == SOC):
        wiringPiSetupGpio()
    elif (mode == WIRINGPI):
        wiringPiSetup()
    else:
        print("'mode' parameter in setupmode() was set to unsupported parameter.")
        sys.exit(1)


def getmode():
    '''
    Get numbering mode used for channel numbers.
    Returns BOARD, BCM, SOC, WIRINGPI or None
    '''
    return _gpio_mode


def setup(channels, direction, pull_up_down=PUD_OFF, initial=None):
    '''
    Set up a GPIO channel or list of channels with a direction and (optional) pull/up down control.
    channel        - A pin number depending on which mode is set.
    direction      - IN or OUT
    [pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN
    [initial]      - Initial value for an output channel"
    '''
    if type(channels) is not list:
        channels = [channels]

    if initial is not None:
        # todo
        print(
            "'initial' parameter in setup() is not yet supported. Set to 'None' or not set.")
    else:
        if direction == OUT:
            for i in channels:
                pinMode(i, OUT)
        elif direction == IN:
            if pull_up_down == PUD_OFF:
                direction = _IN_PULL_OFF
            elif pull_up_down == PUD_DOWN:
                direction = _IN_PULL_DOWN
            elif pull_up_down == PUD_UP:
                direction = _IN_PULL_UP

            for i in channels:
                pinMode(i, direction)


def cleanup(channel=None):
    '''
    Clean up by resetting all GPIO channels that have been used by this program to INPUT
            with no pullup/pulldown and no event detection.
    [channel] - individual channel or list/tuple of channels to clean up.
            Default - clean every channel that has been used.
    '''
    _warning_unsupported_func()


def input(channel):
    '''
    Input from a GPIO channel.  Returns HIGH=1=True or LOW=0=False.
    channel - A pin number depending on which mode is set.
    '''
    return digitalRead(channel)


def output(channels, status):
    '''
    Output to a GPIO channel or list of channels.
    channel - A pin number depending on which mode is set.
    value   - 0/1 or False/True or LOW/HIGH.
    '''
    if type(channels) is not list:
        channels = [channels]

    for i in channels:
        digitalWrite(i, status)


def event_detected(channel):
    '''
    Returns True if an edge has occurred on a given GPIO.
            You need to enable edge detection using add_event_detect() first.
    channel - A pin number depending on which mode is set.
    '''
    gpio = getModePinToGpio(channel)

    return event.edge_event_detected(gpio)


def add_event_callback(channel, callback):
    '''
    Add a callback for an event already defined using add_event_detect().
    channel      - A pin number depending on which mode is set.
    callback     - A callback function.
    '''
    gpio = getModePinToGpio(channel)
    if not callable(callback):
        raise TypeError("Parameter must be callable")

    if not event.gpio_event_added(gpio):
        raise RuntimeError("Add event detection using add_event_detect first "
                           "before adding a callback")

    event.add_edge_callback(gpio, lambda: callback(channel))


def add_event_detect(channel, edge, callback=None, bouncetime=None):
    '''
    Enable edge detection events for a particular GPIO channel.
    channel      - A pin number depending on which mode is set.
    edge         - RISING, FALLING or BOTH.
    [callback]   - A callback function for the event (optional).
    [bouncetime] - Switch bounce timeout in ms for callback.
    '''
    gpio = getModePinToGpio(channel)

    if (not callable(callback)) and callback is not None:
        raise TypeError("Callback Parameter must be callable")

    # if bouncetime is provided, it must be int and greater than 0
    if bouncetime is not None:
        if type(bouncetime) != int:
            raise TypeError("bouncetime must be an integer")

        elif bouncetime < 0:
            raise ValueError("bouncetime must be an integer greater than 0")

    result = event.add_edge_detect(gpio, edge, bouncetime)

    # result == 1 means a different edge was already added for the channel.
    # result == 2 means error occurred while adding edge (thread or event poll)
    if result:
        error_str = None
        if result == 1:
            error_str = "Conflicting edge already enabled for this GPIO " + \
                        "channel"
        else:
            error_str = "Failed to add edge detection"

        raise RuntimeError(error_str)

    if callback is not None:
        event.add_edge_callback(gpio, lambda: callback(channel))


def remove_event_detect(channel):
    '''
    Remove edge detection for a particular GPIO channel.
    channel - A pin number depending on which mode is set.
    '''
    gpio = getModePinToGpio(channel)

    event.remove_edge_detect(gpio)


def wait_for_edge(channel, edge, bouncetime=None, timeout=None):
    '''
    Wait for an edge.  Returns the channel number or None on timeout.
    channel      - A pin number depending on which mode is set.
    edge         - RISING, FALLING or BOTH.
    [bouncetime] - Time allowed between calls to allow for switchbounce.
    [timeout]    - Timeout in ms.
    '''
    gpio = getModePinToGpio(channel)

    # if bouncetime is provided, it must be int and greater than 0
    if bouncetime is not None:
        if type(bouncetime) != int:
            raise TypeError("bouncetime must be an integer")

        elif bouncetime < 0:
            raise ValueError("bouncetime must be an integer greater than 0")

    # if timeout is specified, it must be an int and greater than 0
    if timeout is not None:
        if type(timeout) != int:
            raise TypeError("Timeout must be an integer")

        elif timeout < 0:
            raise ValueError("Timeout must greater than 0")

    result = event.blocking_wait_for_edge(gpio, edge, bouncetime, timeout)

    # If not error, result == channel. If timeout occurs while waiting,
    # result == None. If error occurs, result == -1 means channel is
    # registered for conflicting edge detection, result == -2 means an error
    # occurred while registering event or polling
    if not result:
        return None
    elif result == -1:
        raise RuntimeError("Conflicting edge detection event already exists "
                           "for this GPIO channel")

    elif result == -2:
        raise RuntimeError("Error waiting for edge")

    else:
        return channel


def gpio_function(channel):
    '''
    Return the current GPIO function (IN, OUT, PWM, SERIAL, I2C, SPI).
    channel - A pin number depending on which mode is set.
    '''
    _warning_unsupported_func()


def event_cleanup(gpio=None):
    _warning_unsupported_func()


class PWM(object):
    def __init__(self, channel, frequency_hz):
        _warning_unsupported_func()

    def __del__(self):
        _warning_unsupported_func()

    def start(self, duty_cycle_percent):
        '''
        Start software PWM.
        duty_cycle_percent - the duty cycle (0.0 to 100.0).
        '''
        _warning_unsupported_func()

    def ChangeFrequency(self, frequency_hz):
        '''
        Change the frequency.
        frequency_hz - frequency in Hz (freq > 1.0).
        '''
        _warning_unsupported_func()

    def ChangeDutyCycle(self, duty_cycle_percent):
        '''
        Change the duty cycle.
        duty_cycle_percent - between 0.0 and 100.0.
        '''
        _warning_unsupported_func()

    def stop(self):
        '''
        Stop software PWM.
        '''
        _warning_unsupported_func()

    def _reconfigure(self, frequency_hz, duty_cycle_percent, start=False):
        _warning_unsupported_func()


def setwarnings(enable):
    '''
    Enable or disable warning messages.
    '''
    global _gpio_warnings
    _gpio_warnings = bool(enable)



'''
MIT License

Copyright (c) 2019-2020 Hyeonki Hong <hhk7734@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
'''

from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
import sys
import setuptools
from os import path

BASE_DIR = path.dirname(path.abspath(__file__))
CHANGELOG_PATH = path.join(BASE_DIR, "CHANGELOG")


class get_pybind_include(object):
    """Helper class to determine the pybind11 include path
    The purpose of this class is to postpone importing pybind11
    until it is actually installed, so that the ``get_include()``
    method can be invoked. """

    def __init__(self, user=False):
        self.user = user

    def __str__(self):
        import pybind11
        return pybind11.get_include(self.user)


ext_modules = [
    Extension(
        "Odroid._GPIO",
        sources=["c_src/gpio.cpp"],
        include_dirs=[
            # Path to pybind11 headers
            get_pybind_include(),
            get_pybind_include(user=True)
        ],
        language="c++"
    ),
]

with open(CHANGELOG_PATH, "r") as f:
    version = f.readline()
    version = version.split()
    version = version[1][1:-1]


def has_flag(compiler, flagname):
    """Return a boolean indicating whether a flag name is supported on
    the specified compiler.
    """
    import tempfile
    with tempfile.NamedTemporaryFile('w', suffix='.cpp') as f:
        f.write('int main (int argc, char **argv) { return 0; }')
        try:
            compiler.compile([f.name], extra_postargs=[flagname])
        except setuptools.distutils.errors.CompileError:
            return False
    return True


def cpp_flag(compiler):
    """Return the -std=c++[11/14/17] compiler flag.
    The newer version is prefered over c++11 (when it is available).
    """
    flags = ['-std=c++17', '-std=c++14', '-std=c++11']

    for flag in flags:
        if has_flag(compiler, flag):
            return flag

    raise RuntimeError('Unsupported compiler -- at least C++11 support '
                       'is needed!')


class BuildExt(build_ext):
    """A custom build extension for adding compiler-specific options."""
    c_opts = {
        'msvc': ['/EHsc'],
        'unix': ["-lwiringPi", "-lm", "-lpthread", "-lrt", "-lcrypt"],
    }
    l_opts = {
        'msvc': [],
        'unix': ["-lwiringPi", "-lm", "-lpthread", "-lrt", "-lcrypt"],
    }

    if sys.platform == 'darwin':
        darwin_opts = ['-stdlib=libc++', '-mmacosx-version-min=10.7']
        c_opts['unix'] += darwin_opts
        l_opts['unix'] += darwin_opts

    def build_extensions(self):
        ct = self.compiler.compiler_type
        opts = self.c_opts.get(ct, [])
        link_opts = self.l_opts.get(ct, [])
        if ct == 'unix':
            opts.append('-DVERSION_INFO="%s"' %
                        self.distribution.get_version())
            opts.append(cpp_flag(self.compiler))
            if has_flag(self.compiler, '-fvisibility=hidden'):
                opts.append('-fvisibility=hidden')
        elif ct == 'msvc':
            opts.append('/DVERSION_INFO=\\"%s\\"' %
                        self.distribution.get_version())
        for ext in self.extensions:
            ext.extra_compile_args = opts
            ext.extra_link_args = link_opts
        build_ext.build_extensions(self)


setup(
    version=version,
    ext_modules=ext_modules,
    cmdclass={'build_ext': BuildExt},
)



'''
Copyright (c) 2012-2017 Ben Croston <ben@croston.org>
Copyright (c) 2019 NVIDIA CORPORATION
Copyright (c) 2019 Hyeonki Hong <hhk7734@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
'''
# Python2 has module thread. Renamed to _thread in Python3
try:
    import thread
except:
    import _thread as thread

from select import epoll, EPOLLIN, EPOLLET, EPOLLPRI
from datetime import datetime

try:
    InterruptedError = InterruptedError
except:
    InterruptedError = IOError

import os

# sysfs root
ROOT = "/sys/class/gpio"

# Edge possibilities
NO_EDGE = 0
RISING_EDGE = 1
FALLING_EDGE = 2
BOTH_EDGE = 3

# epoll thread object
_epoll_fd_thread = None

# epoll blocking wait object
_epoll_fd_blocking = None

# dictionary of GPIO class objects. key = gpio number,
# value = GPIO class object
_gpio_event_list = {}

# variable to keep track of thread state
_thread_running = False

# string representations for edges to write to sysfs
_edge_str = ["none", "rising", "falling", "both"]

# lock object for thread
_mutex = thread.allocate_lock()


class _Gpios:
    def __init__(self, gpio, edge=None, bouncetime=None):
        if not os.path.exists(os.path.join(ROOT, "gpio{}".format(gpio))):
            with open(os.path.join(ROOT, "export"), 'w') as f:
                f.write("{}\n".format(gpio))

        self.edge = edge
        self.value_fd = open(ROOT + "/gpio%i" % gpio + "/value", 'r')
        self.initial_thread = True
        self.initial_wait = True
        self.thread_added = False
        self.bouncetime = bouncetime
        self.gpio = gpio
        self.callbacks = []
        self.lastcall = 0
        self.event_occurred = False

    def __del__(self):
        self.value_fd.close()
        del self.callbacks


def add_edge_detect(gpio, edge, bouncetime):
    global _epoll_fd_thread
    gpios = None
    res = gpio_event_added(gpio)

    # event not added
    if not res:
        gpios = _Gpios(gpio, edge, bouncetime)
        _set_edge(gpio, edge)

    # event already added
    elif res == edge:
        gpios = _get_gpio_object(gpio)
        if ((bouncetime is not None and gpios.bouncetime != bouncetime) or
                gpios.thread_added):
            return 1
    else:
        return 1
    # create epoll object for fd if not already open
    if _epoll_fd_thread is None:
        _epoll_fd_thread = epoll()
        if _epoll_fd_thread is None:
            return 2

    # add eventmask and fd to epoll object
    try:
        _epoll_fd_thread.register(gpios.value_fd, EPOLLIN | EPOLLET | EPOLLPRI)
    except IOError:
        remove_edge_detect(gpio)
        return 2

    gpios.thread_added = 1
    _gpio_event_list[gpio] = gpios

    # create and start poll thread if not already running
    if not _thread_running:
        try:
            thread.start_new_thread(_poll_thread, ())
        except RuntimeError:
            remove_edge_detect(gpio)
            return 2
    return 0


def remove_edge_detect(gpio):
    if gpio not in _gpio_event_list:
        return

    if _epoll_fd_thread is not None:
        _epoll_fd_thread.unregister(_gpio_event_list[gpio].value_fd)

    _set_edge(gpio, NO_EDGE)

    if os.path.exists(os.path.join(ROOT, "gpio{}".format(gpio))):
        with open(os.path.join(ROOT, "unexport"), 'w') as f:
            f.write("{}\n".format(gpio))

    _mutex.acquire()
    del _gpio_event_list[gpio]
    _mutex.release()


def add_edge_callback(gpio, callback):
    if gpio not in _gpio_event_list or not _gpio_event_list[gpio].thread_added:
        return

    _gpio_event_list[gpio].callbacks.append(callback)


def edge_event_detected(gpio):
    retval = False
    if gpio in _gpio_event_list:
        _mutex.acquire()
        if _gpio_event_list[gpio].event_occurred:
            _gpio_event_list[gpio].event_occurred = False
            retval = True
        _mutex.release()
        return retval


def gpio_event_added(gpio):
    if gpio not in _gpio_event_list:
        return NO_EDGE

    return _gpio_event_list[gpio].edge


def _get_gpio_object(gpio):
    if gpio not in _gpio_event_list:
        return None
    return _gpio_event_list[gpio]


def _set_edge(gpio, edge):
    edge_path = ROOT + "/gpio%i" % gpio + "/edge"

    with open(edge_path, 'w') as edge_file:
        edge_file.write(_edge_str[edge])


def _get_gpio_obj_key(fd):
    for key in _gpio_event_list:
        if _gpio_event_list[key].value_fd == fd:
            return key
    return None


def _get_gpio_file_object(fileno):
    for key in _gpio_event_list:
        if _gpio_event_list[key].value_fd.fileno() == fileno:
            return _gpio_event_list[key].value_fd
    return None


def _poll_thread():
    global _thread_running

    _thread_running = True
    while _thread_running:
        try:
            events = _epoll_fd_thread.poll(maxevents=1)
            fd = events[0][0]
            _mutex.acquire()

            # check if file object has been deleted or closed from main thread
            fd = _get_gpio_file_object(fd)
            if fd is None or fd.closed:
                _mutex.release()
                continue

            # read file to make sure event is valid
            fd.seek(0)
            if len(fd.read().rstrip()) != 1:
                _thread_running = False
                _mutex.release()
                thread.exit()

            # check key to make sure gpio object has not been deleted
            # from main thread
            key = _get_gpio_obj_key(fd)
            if key is None:
                _mutex.release()
                continue

            gpio_obj = _gpio_event_list[key]

            # ignore first epoll trigger
            if gpio_obj.initial_thread:
                gpio_obj.initial_thread = False
                _gpio_event_list[key] = gpio_obj
                _mutex.release()

            else:
                # debounce the input event for the specified bouncetime
                time = datetime.now()
                time = time.second * 1E6 + time.microsecond
                if (gpio_obj.bouncetime is None or
                        (time - gpio_obj.lastcall >
                         gpio_obj.bouncetime * 1000) or
                        (gpio_obj.lastcall == 0) or gpio_obj.lastcall > time):
                    gpio_obj.lastcall = time
                    gpio_obj.event_occurred = True
                    _gpio_event_list[key] = gpio_obj
                    _mutex.release()
                    for cb_func in gpio_obj.callbacks:
                        cb_func()

        # if interrupted by a signal, continue to start of the loop
        except InterruptedError:
            if _mutex.locked():
                _mutex.release()
            continue
        except AttributeError:
            break
    thread.exit()


def blocking_wait_for_edge(gpio, edge, bouncetime, timeout):
    global _epoll_fd_blocking
    gpio_obj = None
    finished = False
    res = None
    initial_edge = True

    if timeout is None:
        timeout = -1
    else:
        timeout = float(timeout) / 1000

    if gpio in _gpio_event_list:
        if _gpio_event_list[gpio].callbacks:
            return -1

    # check if gpio edge already added. Add if not already added
    added_edge = gpio_event_added(gpio)

    # get existing record
    if added_edge == edge:
        gpio_obj = _get_gpio_object(gpio)
        if (gpio_obj.bouncetime is not None and
                gpio_obj.bouncetime != bouncetime):
            return -1

    # not added. create new record
    elif added_edge == NO_EDGE:
        gpio_obj = _Gpios(gpio, edge, bouncetime)
        _set_edge(gpio, edge)
        _gpio_event_list[gpio] = gpio_obj

    # added_edge != edge. Event is for different edge
    else:
        _mutex.acquire()
        gpio_obj = _get_gpio_object(gpio)
        _set_edge(gpio, edge)
        gpio_obj.edge = edge
        gpio_obj.bouncetime = bouncetime
        gpio_obj.initial_wait = True
        _gpio_event_list[gpio] = gpio_obj
        _mutex.release()

    # create epoll blocking object if not already created
    if _epoll_fd_blocking is None:
        _epoll_fd_blocking = epoll()
        if _epoll_fd_blocking is None:
            return -2

    # register gpio value fd with epoll
    try:
        _epoll_fd_blocking.register(gpio_obj.value_fd, EPOLLIN | EPOLLET |
                                    EPOLLPRI)
    except IOError:
        print("IOError occured while register epoll blocking for GPIO %s"
              % gpio)
        return -2

    while not finished:
        # retry polling if interrupted by signal
        try:
            res = _epoll_fd_blocking.poll(timeout, maxevents=1)
        except InterruptedError:
            continue

        # First trigger is with current state so ignore
        if initial_edge:
            initial_edge = False
            continue

        # debounce input for specified time
        else:
            time = datetime.now()
            time = time.second * 1E6 + time.microsecond
            if ((gpio_obj.bouncetime is None) or
                    (time - gpio_obj.lastcall > gpio_obj.bouncetime * 1000) or
                    (gpio_obj.lastcall == 0) or (gpio_obj.lastcall > time)):
                gpio_obj.lastcall = time
                _mutex.acquire()
                _gpio_event_list[gpio] = gpio_obj
                _mutex.release()
                finished = True

    # check if the event detected was valid
    if res:
        fileno = res[0][0]
        fd = gpio_obj.value_fd
        if fileno != fd.fileno():
            _epoll_fd_blocking.unregister(gpio_obj.value_fd)
            print("File object not found after wait for GPIO %s" % gpio)
            return -2
        else:
            _mutex.acquire()
            fd.seek(0)
            value_str = fd.read().rstrip()
            _mutex.release()
            if len(value_str) != 1:
                _epoll_fd_blocking.unregister(gpio_obj.value_fd)
                print("Length of value string was not 1 for GPIO %s" % gpio)
                return -2

    _epoll_fd_blocking.unregister(gpio_obj.value_fd)

    # 0 if timeout occured - res == []
    # 1 if event was valid
    return int(res != [])


def event_cleanup(gpio=None):
    global _epoll_fd_thread, _epoll_fd_blocking, _thread_running

    _thread_running = False
    if gpio in _gpio_event_list:
        remove_edge_detect(gpio)

    if _gpio_event_list == {}:
        if _epoll_fd_blocking is not None:
            _epoll_fd_blocking.close()
            _epoll_fd_blocking = None

        if _epoll_fd_thread is not None:
            _epoll_fd_thread.close()
            _epoll_fd_thread = None
